//
// Spectre MOSFET model parameters  ---  CMOSV process
// 
// Emperical parameters, best, typical, and worst cases.
// 
// This file requires that Spectre be run with the C-preprocessor (spectre -E).
//
// The following group of models represent MOSFETs over the
// following ranges:
//     6u <= l <= oo
//     6u <= w <= oo
//     process corners = {BEST, TYPICAL, WORST}
// Warnings are issued if these limits are violated.
//
// This model takes 4 parameters, l, w, ls, and ld.  The defaults for each
// of these parameters is 6um.
//
//              +-----------+
//     +--------+           +--------+ ---
//     |        |           |        |  ^
//     |        |           |        |  |
//     |<- ls ->|<--- l --->|<- ld ->|  w
//     |        |           |        |  |
//     |        |           |        |  v
//     +--------+           +--------+ ---
//              +-----------+
//       Source      Gate       Drain
//

#ifndef CMOSV_MODELS_INCLUDED
#define CMOSV_MODELS_INCLUDED

simulator lang=spectre

//
// Define the possible process corners
//
#define BEST	1
#define TYPICAL	2
#define WORST	3

//
// Define the process corner and temperature if not already defined.
//
#ifndef PROCESS_CORNER
#   define PROCESS_CORNER TYPICAL
#endif

//
// Complain if global constants are out-of-range.
//
UnknownCorner paramtest warnif=((PROCESS_CORNER != BEST) && \
				(PROCESS_CORNER != TYPICAL) && \
				(PROCESS_CORNER != WORST)) \
	message="The nmos model does not recognize given PROCESS_CORNER."


//
// NMOS
//

subckt nmos (drain gate source bulk)
    parameters l=6um w=6um ls=6um ld=6um

    //
    // Complain if subcircuit parameters are out-of-range.
    //
    TooShort paramtest warnif=(l < 6um) \
            message="The nmos model is not accurate for lengths less than 6u."
    TooThin paramtest warnif=(w < 6um) \
            message="The nmos model is not accurate for widths less than 6u."

    //
    // The NMOS models
    //
    model nmos_best mos3 type=n vto=1.04 gamma=1.34 phi=.55 nsub=1e15 \
		cgso=290p cgdo=290p cgbo=250p cj=360u tox=700e-10 \
		pb=0.914 js=1e-4 xj=1.2u ld=1.2u wd=0.9u uo=793 bvj=14

    model nmos_typ mos3 type=n vto=1.26 gamma=1.62 phi=.58 nsub=1e15 \
		cgso=370p cgdo=370p cgbo=250p cj=400u tox=750e-10 \
		pb=0.914 js=1e-4 xj=1u ld=0.8u wd=1.2u uo=717 bvj=14

    model nmos_worst mos3 type=n vto=1.48 gamma=1.90 phi=.59 nsub=1e15 \
		cgso=440p cgdo=440p cgbo=250p cj=440u tox=800e-10 \
		pb=0.914 js=1e-4 xj=0.8u ld=0.38u wd=1.5u uo=641 bvj=14

    //
    // Model selection
    //
    if (PROCESS_CORNER == BEST)
	Mnb drain gate source bulk nmos_best l=l w=w ls=ls ld=ld
    else if (PROCESS_CORNER == TYPICAL)
	Mnt drain gate source bulk nmos_typ l=l w=w ls=ls ld=ld
    else if (PROCESS_CORNER == WORST)
	Mnw drain gate source bulk nmos_worst l=l w=w ls=ls ld=ld
    else NoModel paramtest errorif=1 message="Invalid process corner."
ends nmos
    


//
// PMOS
//

subckt pmos (drain gate source bulk)
    parameters l=6um w=6um ls=6um ld=6um

    //
    // Complain if subcircuit parameters are out-of-range.
    //
    TooShort paramtest warnif=(l < 6um) \
            message="The nmos model is not accurate for lengths less than 6u."
    TooThin paramtest warnif=(w < 6um) \
            message="The nmos model is not accurate for widths less than 6u."

    //
    // The PMOS models
    //
    model pmos_best mos3 type=p vto=-0.79 gamma=0.2 phi=.71 nsub=1.7e16 \
		cgso=140p cgdo=140p cgbo=250p cj=80u tox=700e-10 \
		pb=0.605 js=1e-4 xj=0.8u ld=0.9u wd=0.9u uo=245 bvj=14

    model pmos_typ mos3 type=p vto=-1.11 gamma=0.39 phi=.72 nsub=1.7e16 \
		cgso=220p cgdo=220p cgbo=250p cj=100u tox=750e-10 \
		pb=0.605 js=1e-4 xj=0.65u ld=0.5u wd=1.2u uo=206 bvj=14

    model pmos_worst mos3 type=p vto=-1.42 gamma=0.58 phi=.73 nsub=1.7e16 \
		cgso=300p cgdo=300p cgbo=250p cj=120u tox=800e-10 \
		pb=0.605 js=1e-4 xj=0.5u ld=0.1u wd=1.5u uo=167 bvj=14

    //
    // Model selection
    //
    if (PROCESS_CORNER == BEST)
	Mpb drain gate source bulk pmos_best l=l w=w ls=ls ld=ld
    else if (PROCESS_CORNER == TYPICAL)
	Mpt drain gate source bulk pmos_typ l=l w=w ls=ls ld=ld
    else if (PROCESS_CORNER == WORST)
	Mpw drain gate source bulk pmos_worst l=l w=w ls=ls ld=ld
    else NoModel paramtest errorif=1 message="Invalid process corner."
ends pmos
    

//
// DIODE
//
model dnp diode is=3.1e-10 n=1.12 cjo=3.1e-8 pb=.914 m=.5 bvj=45 imax=1000
model dpn diode is=1.3e-10 n=1.05 cjo=9.8e-9 pb=.605 m=.5 bvj=45 imax=1000

#endif
